<?php

/**
 * @file
 * 
 * Contains the callbacks and theme functions for admin pages
 */

/**
 * FAPI callback for settings form generation
 */
function vinfield_settings_form($form_state) {
  $form = array(); //array('#theme' => 'vinfield_settings');
  $settings = variable_get(VF_CONFIG, NULL);
  $defval = NULL;
  
  $form[VF_CONFIG_BASIC_VALIDATION] = array(
    '#type' => 'checkbox',
    '#title' => t('Perform basic VIN validation'),
    '#description' => t('SAE check-digit calculation, length, character set, etc - this can catch typographical errors before sending a lookup request to a provider.  It can also be used standalone to simply record properly formed VIN values.'),
    '#default_value' => isset($settings[VF_CONFIG_BASIC_VALIDATION]) ? $settings[VF_CONFIG_BASIC_VALIDATION] : 1,
  );
  
  $form[VF_CONFIG_LOOKUP_VIN] = array(
    '#type' => 'checkbox',
    '#title' => t('Perform VIN lookup'),
    '#description' => t('Use one of the providers below to look up vehicle information using the given VIN'),
    '#default_value' => isset($settings[VF_CONFIG_LOOKUP_VIN]) ? $settings[VF_CONFIG_LOOKUP_VIN] : 0,
  );

  $form[VF_CONFIG_SAVE_EXTENDED] = array(
    '#type' => 'checkbox',
    '#title' => t('Save extended VIN record data from lookup'),
    '#description' => t('Checking this will store large amounts of information beyond the basic (year, make, model, trim level).  If you need this data, keep this checked but note that your database will be storing large amounts of raw lookup data.'),
    '#default_value' => isset($settings[VF_CONFIG_SAVE_EXTENDED]) ? $settings[VF_CONFIG_SAVE_EXTENDED] : 1,
  );

  $form[VF_CONFIG_REQUIRE_LOOKUP] = array(
    '#type' => 'checkbox',
    '#title' => t('Require successful lookup'),
    '#description' => t('If checked, VIN fails validation if all lookups fail'),
    '#default_value' => isset($settings[VF_CONFIG_REQUIRE_LOOKUP]) ? $settings[VF_CONFIG_REQUIRE_LOOKUP] : 1,
  );

  // Generate list of providers implementing hook_vinfield_provider_info()
  $providers = _vinfield_get_providers();
  $prov_num = count($providers);
  $form[VF_CONFIG_PROVIDERS] = array(
    '#tree' => TRUE,
  );
  
  $prov_tree = &$form[VF_CONFIG_PROVIDERS];
  foreach ($providers as $prov_name => $prov_info) {
    $prov_settings = &$settings[VF_CONFIG_PROVIDERS][$prov_name];
    
    $prov_tree[$prov_name][VF_PROVIDER_TITLE] = array(
      '#value' => check_plain($prov_info[VF_PROVIDER_TITLE]),
    );

    $prov_tree[$prov_name][VF_CONFIG_PROVIDER_ENABLED] = array(
      '#type' => 'checkbox',
      '#default_value' => isset($prov_settings[VF_CONFIG_PROVIDER_ENABLED]) ? $prov_settings[VF_CONFIG_PROVIDER_ENABLED] : 0, 
    );
    
    $prov_tree[$prov_name][VF_CONFIG_PROVIDER_WEIGHT] = array(
      '#type' => 'weight',
      '#delta' => $prov_num,
      '#default_value' => isset($prov_settings[VF_CONFIG_PROVIDER_WEIGHT]) ? $prov_settings[VF_CONFIG_PROVIDER_WEIGHT] : $prov_num, 
    );
  }
  
  $form['submit'] = array(
    '#type' => 'submit', 
    '#value' => t('Save Settings'),
  );

  $form['reset'] = array(
    '#type' => 'submit', 
    '#value' => t('Reset to Defaults'),
    '#submit' => array('vinfield_settings_form_reset'),
  );
  
  return $form;
}

/**
 * FAPI callback for settings form validation
 */
function vinfield_settings_form_validate($form, &$form_state) {
}

/**
 * FAPI callback for settings form submittal
 */
function vinfield_settings_form_submit($form, &$form_state) {
  variable_set(VF_CONFIG, $form_state['values']);
  drupal_set_message(t('VIN Field settings have been saved'));
}

function vinfield_settings_form_reset($form, &$form_state) {
  variable_del(VF_CONFIG);
  drupal_set_message(t('VIN Field settings have been reset to installation defaults'));
}

/**
 * Theme function for settings form
 */
function theme_vinfield_settings_form($form) {
  drupal_add_tabledrag('vinfield-providers', 'order', 'sibling', 'vinfield-element');

  // Header
  $header = array(
    t('Provider'),
    t('Enabled'),
    t('Weight'),
    );

  // Rows for the tabledrag
  $rows = array();

  $prov_tree = &$form[VF_CONFIG_PROVIDERS];
  foreach (element_children($prov_tree) as $prov_name) {
    $prov_tree[$prov_name][VF_CONFIG_PROVIDER_WEIGHT]['#attributes']['class'] = 'vinfield-element';

    $row = array();
    $row[] = drupal_render($prov_tree[$prov_name][VF_PROVIDER_TITLE]);
    $row[] = drupal_render($prov_tree[$prov_name][VF_CONFIG_PROVIDER_ENABLED]);
    $row[] = drupal_render($prov_tree[$prov_name][VF_CONFIG_PROVIDER_WEIGHT]);
    
    $rows[] = array('data' => $row, 'class' => 'draggable');
  }
  
  $output = '';
  $output .= drupal_render($form[VF_CONFIG_BASIC_VALIDATION]);
  $output .= drupal_render($form[VF_CONFIG_LOOKUP_VIN]);
  $output .= drupal_render($form[VF_CONFIG_SAVE_EXTENDED]);
  $output .= drupal_render($form[VF_CONFIG_REQUIRE_LOOKUP]);
  
  $output .= theme('table', $header, $rows, array('id' => 'vinfield-providers'));
  
  $output .= drupal_render($form);

  return $output;
}

